<?
include_once("file.inc");
define("APK_XML_OUTTYPE_FILE_TRUNCATE", 1);
define("APK_XML_OUTTYPE_FILE_APPEND", 2);
define("APK_XML_OUTTYPE_BUFFER", 3);
define("APK_XML_OUTTYPE_SCREEN", 4);
class apk_xml
{
	var $db = NULL;

	function apk_xml($type, $path = false)
	{
		$this->handler = xml_parser_create();

		if($type == APK_XML_OUTTYPE_FILE_TRUNCATE)
		{
			if($path)
			{
				$this->file = new apk_file;
				$this->file->open($path, "w");
			}
		}
		else if($type == APK_XML_OUTTYPE_FILE_APPEND)
		{
			if($path)
			{
				$this->file = new apk_file;
				$this->file->open($path, "w");
			}
		}

		$this->outtype = $type;
	}

	function cache_path($path)
	{
		$this->cachepath = $cpath = dirname($path)."/.".basename($path).".cache";
		return $cpath;
	}

	function parse_file($path)
	{
		$this->inputpath = $path;
		$cpath=$this->cache_path($path);

		if(!file_exists($cpath) || (filemtime($path) > filemtime($cpath)))
		{
			$data = join("", file($path));
			$this->parse_data($data);
		}
		else
		{
			$this->db = unserialize(join("", file($cpath)));
		}
	}

	function parse_data($data)
	{
		if(!xml_parse_into_struct($this->handler, $data, $this->values, $this->index))
		{
			echo sprintf("XML error: %s at line %d\n",
			xml_error_string(xml_get_error_code($this->handler)),
			xml_get_current_line_number($this->handler));
		}
		xml_parser_free($this->handler);
	}

	function build_struct()
	{
		if(!$this->db)
		{
			foreach($this->index as $k=>$val)
			{
				$tmp = array();
				for($i = 1 ; $i < count($val) ; $i++)
				{
					$offset = $val[$i-1]+1;
					$len = $val[$i] - $offset;
					$tmp = array_merge($tmp, $this->parse_entry($offset, $len));
				}
				$this->db = $tmp;
				$this->db['NAME'] = $k;
				break;
			}

			$fp = fopen($this->cachepath, "w");

			if($fp)
			{
				fwrite($fp, serialize($this->db));
				fclose($fp);
			}
		}

		return $this->db;
	}

	function parse_entry($offset, $len)
	{
		$result = array();
		$vals = array_slice($this->values, $offset, $len);

		$count=0;
		for($i = 0 ; $i < count($vals) ; $i++)
		{
			$name = $vals[$i]['tag'];
			$level = $vals[$i]['level'];
			if(($vals[$i]['type'] != 'open') && ($vals[$i]['type'] != 'cdata'))
			{
				if($vals[$i]['value'])
					$tmp['value'] = trim($vals[$i]['value']);
				else
					$tmp['attributes'] = $vals[$i]['attributes'];

				$result[$count][$name] = array_merge($result[$count][$name], $tmp);
				$count++;
			}
			else if($vals[$i]['type'] != 'cdata')
			{
				$result[$count][$name]['attributes'] = $vals[$i]['attributes'];
				$i++;
				$start=$i;
				while($i < count($vals))
				{
					if(($vals[$i]['type'] == 'close') &&
						($vals[$i]['tag'] == $name) &&
						($vals[$i]['level'] == $level)) break;
					$i++;
				}

				$result[$count][$name]['sub'] = $this->parse_entry($start+$offset, $i-$start);
				$count++;
			}
		}

		return $result;
	}

	function parse_sub($vals, $start, $end)
	{
		for($i = $start ; $i < $end ; $i++)
		{
			if($vals[$i]['type'] == 'cdata')
			{
				$i++;
				continue;
			}
			$sname = $vals[$i]['tag'];
			if($vals[$i]['value'])
				$result[$sname]['value'] = trim($vals[$i]['value']);
			else
				$result[$sname]['attributes'] = $vals[$i]['attributes'];
		}
	}

	function dump()
	{
		$indent = "   ";
		$this->append_xmlstr("<?xml version=\"1.0\"?>\n<".$this->db['NAME'].">\n");

		for($i = 0 ; $i < (count($this->db) - 1) ; $i++)
		{
			foreach($this->db[$i] as $key=>$val)
			{
				$this->dump_tag($key, $val, $indent);
			}
		}
		$this->append_xmlstr("</".$this->db['NAME'].">\n");

		if($this->outtype == APK_XML_OUTTYPE_BUFFER)
			return $this->string;
	}

	function dump_tag($key, $val, &$indent)
	{
		$this->append_xmlstr("$indent<$key");
		if($val['attributes'])
		{
			foreach($val['attributes'] as $aname=>$aval)
			{
				$this->append_xmlstr(" $aname=\"$aval\"");
			}
		}

		if($val['value'])
		{
			$this->append_xmlstr(">".$val['value']."</$key>\n");
		}
		else if($val['sub'])
		{
			$this->append_xmlstr(">\n");
			$indent .= "   ";
			for($i = 0 ; $i < count($val['sub']) ; $i++)
			{
				foreach($val['sub'][$i] as $k=>$v)
				{
					$this->dump_tag($k, $v, $indent);
				}
			}
			$indent = substr($indent, 0, -3);
			$this->append_xmlstr("$indent</$key>\n");
		}
		else
			$this->append_xmlstr(" />\n");
	}

	function append_xmlstr($str)
	{
		switch($this->outtype)
		{
			case APK_XML_OUTTYPE_BUFFER:
				$this->string .= $str;
			break;
			case APK_XML_OUTTYPE_FILE_TRUNCATE:
			case APK_XML_OUTTYPE_FILE_APPEND:
				$this->file->write($str);
			break;
			case APK_XML_OUTTYPE_SCREEN:
				print($str);
		}
	}
}
?>
